import { Body, Controller, Get, Post, Put, Query } from '@nestjs/common';
import { ApiBody, ApiOperation, ApiQuery, ApiTags } from '@nestjs/swagger';
import { Serealize } from '../interceptors/serialize.interceptor';
import { InstitucionInfraccionService } from './institucion-infraccion.service';
import { IdInstitucionDto } from '../dto/id-institucion.dto';
import { CreateInstitucionInfraccionDto } from './dto/input/create.dto';
import { UpdateInstitucionInfraccionDto } from './dto/input/update.dto';
import { InfraccionOutputDto } from './dto/output/infraccion.dto';
import { InstitucionInfraccionOutputDto } from './dto/output/institucion-infraccion.dto';

@Controller('institucion-infraccion')
@ApiTags('institucion-infraccion')
export class InstitucionInfraccionController {
  constructor(
    private institucionInfraccionService: InstitucionInfraccionService,
  ) {}

  @Post()
  @ApiOperation({ description: 'Endpoint que crea una infracción.' })
  @ApiBody({
    description: 'Es obligatorio mandar la variable infraccion.',
    examples: { ejemplo: { value: { infraccion: '' } } },
  })
  create(@Body() body: CreateInstitucionInfraccionDto) {
    return this.institucionInfraccionService.create(body.infraccion);
  }

  @Serealize(InfraccionOutputDto)
  @Get()
  @ApiOperation({ description: 'Endpoint que retorna todas las infracciones.' })
  get() {
    return this.institucionInfraccionService.findAll();
  }

  @Serealize(InstitucionInfraccionOutputDto)
  @Get('infracciones')
  @ApiOperation({
    description: 'Endpoint que retorna las infracciones de una institución.',
  })
  @ApiQuery({
    description: 'Id de la institución.',
    name: 'id_institucion',
    type: 'string',
  })
  infracciones(@Query() query: IdInstitucionDto) {
    return this.institucionInfraccionService.findAllByIdInstitucion(
      parseInt(query.id_institucion),
    );
  }

  @Put()
  @ApiOperation({
    description: 'Endpoint que actualiza los días de multa de una infracción.',
  })
  @ApiBody({
    description: 'Las dos variables son obligatorias.',
    examples: {
      ejemplo: { value: { id_institucion_infraccion: 603, dias_multa: 7 } },
    },
  })
  update(@Body() body: UpdateInstitucionInfraccionDto) {
    return this.institucionInfraccionService.update(body);
  }
}
