import { Body, Controller, Get, Post, Put, Query } from '@nestjs/common';
import { ApiBody, ApiOperation, ApiQuery, ApiTags } from '@nestjs/swagger';
import { Serealize } from '../interceptors/serialize.interceptor';
import { InstitucionProgramaService } from './institucion-programa.service';
import { IdInstitucionDto } from 'src/dto/id-institucion.dto';
import { CreateProgramaDto } from './dto/input/create.dto';
import { UpdateProgramaDto } from './dto/input/update.dto';
import { InstitucionProgramaOutputDto } from './dto/output/institucion-programa.dto';
import { InstitucionProgramaMinOutputDto } from './dto/output/institucion-programa-min.dto';
import { ProgramaOutputDto } from './dto/output/programa.dto';

@Controller('institucion-programa')
@ApiTags('institucion-programa')
export class InstitucionProgramaController {
  constructor(private institucionProgramaService: InstitucionProgramaService) {}

  @Post()
  @ApiOperation({ description: 'Endpoint que crea un programa nuevo.' })
  @ApiBody({
    description: 'Es obligatorio mandar la varible programa.',
    examples: { ejemplo: { value: { programa: '' } } },
  })
  create(@Body() body: CreateProgramaDto) {
    return this.institucionProgramaService.create(body.programa);
  }

  @Serealize(ProgramaOutputDto)
  @Get()
  @ApiOperation({ description: 'Endpoint que retorna todos los programas.' })
  get() {
    return this.institucionProgramaService.findAllProgramas();
  }

  @Serealize(InstitucionProgramaOutputDto)
  @Get('programas')
  @ApiOperation({
    description: 'Endpoint que retorna todos los programas de una institución.',
  })
  @ApiQuery({
    description: 'Id de la institución.',
    name: 'id_institucion',
    type: 'string',
  })
  programas(@Query() query: IdInstitucionDto) {
    return this.institucionProgramaService.findAllByIdInstitucion(
      parseInt(query.id_institucion),
    );
  }

  @Serealize(InstitucionProgramaMinOutputDto)
  @Get('programas-mostrar')
  @ApiOperation({
    description:
      'Endpoint que retorna todos los programas de una institución que tengan el campo mostrar en true.',
  })
  @ApiQuery({
    description: 'Id de la institución.',
    name: 'id_institucion',
    type: 'string',
  })
  programasMostrar(@Query() query: IdInstitucionDto) {
    return this.institucionProgramaService.findAllByIdInstitucionMostrar(
      parseInt(query.id_institucion),
    );
  }

  @Put()
  @ApiOperation({
    description:
      'Endpoint que actualiza la información de una institución programa.',
  })
  @ApiBody({
    description: 'Es obligatorio mandar la varible programa.',
    examples: {
      ejemplo: { value: { id_institucion_programa: 130, mostrar: true } },
    },
  })
  update(@Body() body: UpdateProgramaDto) {
    return this.institucionProgramaService.update(body);
  }
}
