import { Body, Controller, Get, Post, Put, Query } from '@nestjs/common';
import { ApiBody, ApiOperation, ApiQuery, ApiTags } from '@nestjs/swagger';
import { Serealize } from '../interceptors/serialize.interceptor';
import { InstitucionTipoCarritoService } from './institucion-tipo-carrito.service';
import { IdInstitucionDto } from '../dto/id-institucion.dto';
import { CreateInstitucionTipoCarritoDto } from './dto/input/create.dto';
import { UpdateInstitucionTipoCarritoDto } from './dto/input/update.dto';
import { TipoCarritoOutputDto } from './dto/output/tipo-carrito.dto';
import { InstitucionTiposCarritoOutputDto } from './dto/output/institucion-tipos-carrito.dto';
import { InstitucionTiposCarritoMostarOutputDto } from './dto/output/institucion-tipos-carrito-mostar.dto';

@Controller('institucion-tipo-carrito')
@ApiTags('institucion-tipo-carrito')
export class InstitucionTipoCarritoController {
  constructor(
    private institucionTipoCarritoService: InstitucionTipoCarritoService,
  ) {}

  @Post()
  @ApiOperation({ description: 'Endpoint que crea un tipo de carrito.' })
  @ApiBody({
    description: 'Ambas variables son obligatorias.',
    examples: { ejemplo: { value: { letra: '', tipo_carrito: '' } } },
  })
  create(@Body() body: CreateInstitucionTipoCarritoDto) {
    return this.institucionTipoCarritoService.create(
      body.letra,
      body.tipo_carrito,
    );
  }

  @Serealize(TipoCarritoOutputDto)
  @Get()
  @ApiOperation({
    description: 'Endpoint que retorna todos los tipos de carrito.',
  })
  get() {
    return this.institucionTipoCarritoService.findAll();
  }

  @Serealize(InstitucionTiposCarritoOutputDto)
  @Get('tipos-carrito')
  @ApiOperation({
    description:
      'Endpoint que retorna todos los tipos de carrito de una institución.',
  })
  @ApiQuery({
    description: 'Id de la institución.',
    name: 'id_institucion',
    type: 'string',
  })
  tiposCarrito(@Query() query: IdInstitucionDto) {
    return this.institucionTipoCarritoService.findAllByIdInstitucion(
      parseInt(query.id_institucion),
    );
  }

  @Serealize(InstitucionTiposCarritoMostarOutputDto)
  @Get('tipos-carrito-mostrar')
  @ApiOperation({
    description:
      'Endpoint que retorna todos los tipos de carrito de una institución que tengan el campo mostrar en true.',
  })
  @ApiQuery({
    description: 'Id de la institución.',
    name: 'id_institucion',
    type: 'string',
  })
  tiposCarritoMostrar(@Query() query: IdInstitucionDto) {
    return this.institucionTipoCarritoService.findAllByIdInstitucionMostar(
      parseInt(query.id_institucion),
    );
  }

  @Put()
  @ApiOperation({
    description:
      'Endpoint que actualiza el campo mostrar de una institución tipo carrito.',
  })
  @ApiBody({
    description: 'Ambas variables son obligatorias.',
    examples: {
      ejemplo: { value: { id_institucion_tipo_carrito: 130, mostrar: true } },
    },
  })
  update(@Body() body: UpdateInstitucionTipoCarritoDto) {
    return this.institucionTipoCarritoService.update(body);
  }
}
