import { Body, Controller, Get, Put, Query } from '@nestjs/common';
import { ApiBody, ApiOperation, ApiQuery, ApiTags } from '@nestjs/swagger';
import { Serealize } from '../interceptors/serialize.interceptor';
import { InstitucionService } from './institucion.service';
import { IdInstitucionDto } from '../dto/id-institucion.dto';
import { UpdateInstitucionDto } from './dto/input/update.dto';
import { InstitucionOutputDto } from './dto/output/institucion.dto';
import { InstitucionesOutputDto } from './dto/output/instituciones.dto';
import { InstitucionMinOutputDto } from './dto/output/institucion-min.dto';

@Controller('institucion')
@ApiTags('institucion')
export class InstitucionController {
  constructor(private institucionService: InstitucionService) {}

  @Serealize(InstitucionMinOutputDto)
  @Get()
  @ApiOperation({
    description: 'Endpoint que retorna todas las instituciones.',
  })
  get() {
    return this.institucionService.findAll();
  }

  @Serealize(InstitucionOutputDto)
  @Get('institucion')
  @ApiOperation({
    description: 'Endpoint que retorna la información de una institución.',
  })
  @ApiQuery({
    description: 'Id de la institución.',
    name: 'id_institucion',
    type: 'string',
  })
  institucion(@Query() query: IdInstitucionDto) {
    return this.institucionService.findById(parseInt(query.id_institucion));
  }

  @Serealize(InstitucionesOutputDto)
  @Get('activas')
  @ApiOperation({
    description: 'Endpoint que retorna todas las instituciones activas.',
  })
  activas() {
    return this.institucionService.findAllActivos();
  }

  @Put()
  @ApiOperation({
    description: 'Endpoint que actualiza la información de una institución.',
  })
  @ApiBody({
    description:
      'Todas las variables a excepción de id_institucion son opcionales.',
    examples: {
      ejemplo: {
        value: {
          id_institucion: 200,
          activo: true,
          dias_multa_retraso: 7,
          dias_multa_retrtiempo_entregaaso: 7,
          tiempo_prestamo: 7,
          tiempo_recoger: 7,
        },
      },
    },
  })
  update(@Body() body: UpdateInstitucionDto) {
    return this.institucionService.update(body);
  }
}
