import { Body, Controller, Get, Post, Put, Query } from '@nestjs/common';
import { ApiBody, ApiOperation, ApiQuery, ApiTags } from '@nestjs/swagger';
import { PrestamoService } from './prestamo.service';
import { PrestamoPedirDto } from './dto/prestamo-pedir.dto';
import { PrestamoActivosDto } from './dto/prestamo-activos.dto';
import { PrestamoIdUsuarioDto } from './dto/prestamo-id-usuario.dto';
import { PrestamoIdPrestamoDto } from './dto/prestamo-id-prestamo.dto';
import { PrestamoNumeroInventarioDto } from './dto/prestamo-numero-inventario.dto';
import { PrestamoHistorialEquipoDto } from './dto/prestamo-historial-equipo.dto';
import { PrestamoHistorialUsuarioDto } from './dto/prestamo-historial-usuario.dto';
import { PrestamoHistorialDto } from './dto/prestamo-historial.dto';
import { PrestamoUpdateOperadorDto } from './dto/prestamo-update-operador.dto';
import { PrestamoCancelarUsuarioDto } from './dto/prestamo-cancelar-usuario.dto';

@Controller('prestamo')
@ApiTags('prestamo')
export class PrestamoController {
  constructor(private prestamoService: PrestamoService) {}

  @Get('activos')
  @ApiOperation({
    description:
      'Endpoint que retorna 25 prestamos activos dependiendo de la página en la que se encuentra el operador y sus filtros.',
  })
  @ApiQuery({
    description: 'Página en la que se encuentra el usuario.',
    name: 'pagina',
    type: 'string',
  })
  @ApiQuery({
    description: 'Carrito que se quiere usar como filtro.',
    name: 'carrito',
    type: 'string',
    required: false,
  })
  @ApiQuery({
    description: 'Equipo que se quiere usar como filtro.',
    name: 'equipo',
    type: 'string',
    required: false,
  })
  @ApiQuery({
    description: 'Id de la institución que se quiere usar como filtro.',
    name: 'id_institucion',
    type: 'string',
    required: false,
  })
  @ApiQuery({
    description: 'Id del modulo que se quiere usar como filtro.',
    name: 'id_modulo',
    type: 'string',
    required: false,
  })
  @ApiQuery({
    description: 'Id del préstamo que se quiere usar como filtro.',
    name: 'id_prestamo',
    type: 'string',
    required: false,
  })
  @ApiQuery({
    description: 'Id del tipo de carrito que se quiere usar como filtro.',
    name: 'id_tipo_carrito',
    type: 'string',
    required: false,
  })
  @ApiQuery({
    description: 'Id del tipo de usuario que se quiere usar como filtro.',
    name: 'id_tipo_usuario',
    type: 'string',
    required: false,
  })
  @ApiQuery({
    description: 'Usuario que se quiere usar como filtro.',
    name: 'usuario',
    type: 'string',
    required: false,
  })
  activos(@Query() query: PrestamoActivosDto) {
    return this.prestamoService.findAll(query);
  }

  @Put('cancelar-operador')
  @ApiOperation({
    description: 'Endpoint que cancela un prestamo por parte del operador.',
  })
  @ApiBody({
    description: 'Ambas variables son obligatorias.',
    examples: { ejemplo: { value: { id_prestamo: 1, id_operador: 1 } } },
  })
  cancelarOperador(@Body() body: PrestamoUpdateOperadorDto) {
    return this.prestamoService.cancelarOperador(
      body.id_prestamo,
      body.id_operador,
    );
  }

  @Put('cancelar-usuario')
  @ApiOperation({
    description: 'Endpoint que cancela el prestamo del usuario.',
  })
  @ApiBody({
    description: 'La variable id_prestamo es obligatoria.',
    examples: { ejemplo: { value: { id_prestamo: 1 } } },
  })
  cancelarUsuario(@Body() body: PrestamoCancelarUsuarioDto) {
    return this.prestamoService.cancelarUsuario(body.id_prestamo);
  }

  @Put('entregar')
  @ApiOperation({
    description: 'Endpoint que actualiza el status del equipo a "En uso".',
  })
  @ApiBody({
    description: 'Ambas variables son obligatorias.',
    examples: { ejemplo: { value: { id_prestamo: 1, id_operador: 4 } } },
  })
  entregar(@Body() body: PrestamoUpdateOperadorDto) {
    return this.prestamoService.entregar(body.id_prestamo, body.id_operador);
  }

  @Get('historial')
  @ApiOperation({
    description:
      'Endpoint que retorna 25 prestamos dependiendo de la página en la que se encuentra el operador y sus filtros.',
  })
  @ApiQuery({
    description: 'Página en la que se encuentra el usuario.',
    name: 'pagina',
    type: 'string',
  })
  @ApiQuery({
    description: 'Carrito que se quiere usar como filtro.',
    name: 'carrito',
    type: 'string',
    required: false,
  })
  @ApiQuery({
    description: 'Equipo que se quiere usar como filtro.',
    name: 'equipo',
    type: 'string',
    required: false,
  })
  @ApiQuery({
    description: 'Fecha inicio que se quiere usar como filtro.',
    name: 'fechaInicio',
    type: 'string',
    required: false,
  })
  @ApiQuery({
    description: 'Fecha fin se quiere usar como filtro.',
    name: 'fechaFin',
    type: 'string',
    required: false,
  })
  @ApiQuery({
    description: 'Id de la institución que se quiere usar como filtro.',
    name: 'id_institucion',
    type: 'string',
    required: false,
  })
  @ApiQuery({
    description: 'Id del modulo que se quiere usar como filtro.',
    name: 'id_modulo',
    type: 'string',
    required: false,
  })
  @ApiQuery({
    description: 'Id del préstamo que se quiere usar como filtro.',
    name: 'id_prestamo',
    type: 'string',
    required: false,
  })
  @ApiQuery({
    description: 'Id del tipo de carrito que se quiere usar como filtro.',
    name: 'id_tipo_carrito',
    type: 'string',
    required: false,
  })
  @ApiQuery({
    description: 'Id del tipo de usuario que se quiere usar como filtro.',
    name: 'id_tipo_usuario',
    type: 'string',
    required: false,
  })
  @ApiQuery({
    description: 'Usuario que se quiere usar como filtro.',
    name: 'usuario',
    type: 'string',
    required: false,
  })
  historial(@Query() query: PrestamoHistorialDto) {
    return this.prestamoService.findAll(query);
  }

  @Get('historial-equipo')
  @ApiOperation({
    description:
      'Endpoint que retorna 25 prestamos de un usuario dependiendo de la página en la que se encuentra el operador y sus filtros.',
  })
  @ApiQuery({
    description: 'Página en la que se encuentra el usuario.',
    name: 'pagina',
    type: 'string',
  })
  @ApiQuery({
    description: 'Id del equipo.',
    name: 'id_equipo',
    type: 'string',
  })
  historialEquipo(@Query() query: PrestamoHistorialEquipoDto) {
    return this.prestamoService.findAllByIdEquipo(
      parseInt(query.id_equipo),
      parseInt(query.pagina),
    );
  }

  @Get('historial-usuario')
  @ApiOperation({
    description:
      'Endpoint que retorna 25 prestamos de un equipo dependiendo de la página en la que se encuentra el operador y sus filtros.',
  })
  @ApiQuery({
    description: 'Página en la que se encuentra el usuario.',
    name: 'pagina',
    type: 'string',
  })
  @ApiQuery({
    description: 'Id del usuario.',
    name: 'id_usuario',
    type: 'string',
  })
  historialUsuario(@Query() query: PrestamoHistorialUsuarioDto) {
    return this.prestamoService.findAllByIdUsuario(
      parseInt(query.id_usuario),
      parseInt(query.pagina),
    );
  }

  @Post()
  @ApiOperation({ description: 'Endpoint que crea un prestamo nuevo.' })
  @ApiBody({
    description:
      'Todas las variables a excepción de id_programa y id_tipo_entrada son obligatorios.',
    examples: {
      ejemplo: {
        value: {
          id_usuario: 1,
          id_modulo: 1,
          id_tipo_carrito: 1,
          id_programa: 1,
          id_tipo_entrada: 1,
        },
      },
    },
  })
  pedir(@Body() body: PrestamoPedirDto) {
    return this.prestamoService.create(
      body.id_usuario,
      body.id_modulo,
      body.id_tipo_carrito,
      body.id_programa,
      body.id_tipo_entrada,
    );
  }

  @Get('prestamo-id-prestamo')
  @ApiOperation({ description: 'Endpoint que retorna un préstamo por su id.' })
  @ApiQuery({ description: '', name: '', type: '' })
  prestamoIdPrestamo(@Query() query: PrestamoIdPrestamoDto) {
    return this.prestamoService.findById(parseInt(query.id_prestamo));
  }

  @Get('prestamo-id-usuario')
  @ApiOperation({
    description:
      'Endpoint que retorna un préstamo activo por el id del usuario.',
  })
  @ApiQuery({
    description: 'Id del usuario.',
    name: 'id_usuario',
    type: 'string',
  })
  prestamoIdUsuario(@Query() query: PrestamoIdUsuarioDto) {
    return this.prestamoService.findByIdUsuario(parseInt(query.id_usuario));
  }

  @Get('prestamo-numero-inventario')
  @ApiOperation({
    description:
      'Endpoint que retorna un préstamo activo por el número de inventario de un equipo.',
  })
  @ApiQuery({
    description: 'Id de la institución.',
    name: 'id_institucion',
    type: 'string',
  })
  @ApiQuery({
    description: 'El número de inventario del equipo.',
    name: 'numero_inventario',
    type: 'string',
  })
  prestamoNumeroInventario(@Query() query: PrestamoNumeroInventarioDto) {
    return this.prestamoService.findByNumeroInventario(
      parseInt(query.id_institucion),
      query.numero_inventario,
    );
  }

  @Put('regresar')
  @ApiOperation({ description: 'Endpoint que desactiva un préstamo.' })
  @ApiBody({
    description: 'Ambas variables son obligatorias.',
    examples: { ejemplo: { value: { id_prestamo: 1, id_operador: 4 } } },
  })
  regresar(@Body() body: PrestamoUpdateOperadorDto) {
    return this.prestamoService.regresar(body.id_prestamo, body.id_operador);
  }

  // @Get('reporte')
  // reporte() {}
}
