import { Body, Controller, Get, Post } from '@nestjs/common';
import { ApiBody, ApiOperation, ApiTags } from '@nestjs/swagger';
import { Serealize } from '../interceptors/serialize.interceptor';
import { TipoUsuarioService } from './tipo-usuario.service';
import { TipoUsuarioUpdateDto } from './dto/input/tipo-usuario-update.dto';
import { TipoUsuarioOutputDto } from './dto/output/tipo-usuario.dto';

@Controller('tipo-usuario')
@ApiTags('tipo-usuario')
export class TipoUsuarioController {
  constructor(private tipoUsuarioService: TipoUsuarioService) {}

  @Post()
  @ApiOperation({ description: 'Endpoint que crea un nuevo tipo de usuario.' })
  @ApiBody({
    description: 'Es obligatoria mandar el campo tipo_usuario.',
    examples: { ejemplo: { value: { tipo_usuario: '' } } },
  })
  create(@Body() body: TipoUsuarioUpdateDto) {
    return this.tipoUsuarioService.create(body.tipo_usuario);
  }

  @Serealize(TipoUsuarioOutputDto)
  @Get()
  @ApiOperation({
    description: 'Endpoint que retorna todos los tipos de usuario.',
  })
  get() {
    return this.tipoUsuarioService.findAll();
  }
}
